﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Wintellect.PowerCollections;
using Ecng.Collections;
using Ecng.Common;
using Ecng.Serialization;
using Ecng.Xaml;
using StockSharp.BusinessEntities;
using StockSharp.Algo;
using StockSharp.Algo.Candles;
using StockSharp.Algo.Storages;
using StockSharp.Algo.Testing;

namespace SampleCandleStorageTesting
{
    public class HistoryCandleSource : ICandleSource
    {
        public BaseTrader Trader { get; private set; }

        public HistoryCandleSource(EmulationTrader trader)
        {
            if (trader == null)
                throw new ArgumentNullException("trader");

            Trader = trader;

            //Trader.MarketTimeChanged += OnMarketTimeChanged;
            Trader.NewTrades += OnNewTrades;

            //Trader.NewHistoryCandles += OnNewHistoryCandles;
            //From = Trader.MarketTime.Date - TimeSpan.FromDays(5); 
        }

        public string HistoryPath { get; set; }
        public DateTime DateFrom { get; set; }
        public DateTime DateTo { get; set; }
        
        private List<CandleToken> _tokens = new List<CandleToken>();
        private Dictionary<Triple<Type, Security, object>, CandleToken> _tokensType = new Dictionary<Triple<Type, Security, object>, CandleToken>();

        private DateTime _currentDay = new DateTime();
        private IEnumerable<Candle> _dayCandles;
        private Candle _nextFistCandle;
        private Candle _nextSecondCandle;
        private int _lastCandleStarted;
        private int _lastCandleFinished;
        private int _nextCandlePos;


        IEnumerable<CandleToken> ICandleSource.Tokens
        {
            get { return _tokens.AsEnumerable(); }            
        }
        
        private Action<CandleToken, IEnumerable<Candle>> _candlesStarted;
        
        event Action<CandleToken, IEnumerable<Candle>> ICandleSource.CandlesStarted
        {
            add { _candlesStarted += value; }
            remove { _candlesStarted -= value; }
        }
        
        event Action<CandleToken, IEnumerable<Candle>> ICandleSource.CandlesChanged
        {
            add { }
            remove { }
        }

        private Action<CandleToken, IEnumerable<Candle>> _candlesFinished;
        
        event Action<CandleToken, IEnumerable<Candle>> ICandleSource.CandlesFinished
        {
            add { _candlesFinished += value; }
            remove { _candlesFinished -= value; }
        }
        
        event Action<Exception> ICandleSource.ProcessDataError
        {
            add { }
            remove { }
        }
        
        CandleToken ICandleSource.GetToken(Type candleType, Security security, object arg)
        {
            CheckTypes(candleType, arg);

            return _tokensType.TryGetValue(new Triple<Type, Security, object>(candleType, security, arg));            
        }

        // для поддержки остальных типов надо доделать IsCandleStarted, IsCandleFinished, LoadDayCandles, CheckTypes, IsSupport, Register, GetToken

        bool ICandleSource.IsSupport(Type candleType, Security security, object arg)
        {
            return candleType == typeof(TimeFrameCandle) && arg is TimeSpan;            
        }

        CandleToken ICandleSource.Register(Type candleType, Security security, object arg)
        {
            if (((ICandleSource)this).GetToken(candleType, security, arg) != null)
                throw new ArgumentException("Группировка свечек уже зарегистрирована ранее с переданными параметрами.");
            
            CheckTypes(candleType, arg);
            
            var token = new CandleToken<TimeFrameCandle, TimeSpan>(security, (TimeSpan) arg);
            _tokens.Add(token);
            _tokensType.Add(new Triple<Type, Security, object>(candleType, security, arg), token);

            return token;
        }
        
        void ICandleSource.UnRegister(CandleToken token)
        {
            _tokens.Remove(token);
            _tokensType.Remove(new Triple<Type, Security, object>(token.CandleType, token.Security, token.Arg));
        }

        private static void CheckTypes(Type candleType, object arg)
        {
            if (candleType == null)
                throw new ArgumentNullException("candleType");
            
            if (candleType != typeof(TimeFrameCandle))
                throw new ArgumentException("HistoryCandle поддерживает свечки только типа {0}.".Put(typeof(TimeFrameCandle)), "candleType");
            
            if (!(arg is TimeSpan))
                throw new ArgumentException("Аргумент {0} должен быть типа {1}.".Put(arg, typeof(TimeSpan)), "arg");
        }        

        private void CandleStarted(CandleToken token, Candle candle)
        {            
            _candlesStarted.SafeInvoke(token, new Candle[1] { candle });
        }

        private void CandleFinished(CandleToken token, Candle candle)
        {         
            _candlesFinished.SafeInvoke(token, new Candle[1] { candle });
        }

        private void LoadDayCandles()
        {
            var storage = new TradingStorage(new InMemoryStorage());
            string CandleTypeText = "TimeFrame";
            var token = _tokens.First();

            storage.BasePath = Path.Combine(HistoryPath, "candles_" + CandleTypeText + "Candles" + "_" + ((TimeSpan)token.Arg).ToString().Replace(':', '_'));

            var marketDataStorage = storage.GetCandleStorage<TimeFrameCandle, TimeSpan>(token.Security, (TimeSpan)token.Arg);

            DateTime date;

            if (_currentDay == new DateTime())
                date = Trader.MarketTime.Date;
            else
                date = _currentDay.AddDays(1);

            _dayCandles = marketDataStorage.Load(date, date + TimeSpan.FromTicks(TimeSpan.TicksPerDay - 1));
            _currentDay = date;

            _nextCandlePos = 0;
            _lastCandleStarted = -1;
            _lastCandleFinished = -1;
            _nextFistCandle = null;
            _nextSecondCandle = null;
        }

        private void ProcessNextCandles(CandleToken token, Trade trade)
        {
            if (trade.Time.Date != _currentDay.Date)
            {
                if (_nextFistCandle != null)
                {
                    this.CandleFinished(token, _nextFistCandle);                    
                }

                LoadDayCandles();
            }

            if (_nextFistCandle == null)
                LoadNextCandle();
            if (_nextFistCandle == null)
                return;

            if (_nextCandlePos - 1 > _lastCandleStarted && IsCandleStarted(token, _nextFistCandle, trade))
            {
                _lastCandleStarted = _nextCandlePos - 1;
                CandleStarted(token, _nextFistCandle);
            }

            if (_nextCandlePos - 1 > _lastCandleFinished && IsCandleFinished(token, _nextFistCandle, trade))            
            {
                CandleFinished(token, _nextFistCandle);

                // Вызываем метод еща раз для вызова CandleStarted следующей свечи (если необходимо)
                if (_nextSecondCandle != null && (IsCandleStarted(token, _nextSecondCandle, trade) || IsCandleFinished(token, _nextSecondCandle, trade)))
                {
                    LoadNextCandle();
                    ProcessNextCandles(token, trade);
                }
                else
                {
                    LoadNextCandle();
                }
            }
        }

        private bool IsCandleStarted(CandleToken token, Candle candle, Trade trade)
        {
            if (token.CandleType == typeof(TimeFrameCandle))
                return candle.Time <= trade.Time;

            return false;
        }

        private bool IsCandleFinished(CandleToken token, Candle candle, Trade trade)
        {
            if (token.CandleType == typeof(TimeFrameCandle))
                return candle.Time.Add((TimeSpan)token.Arg) <= trade.Time;
            // Для остальных типов свечей надо либо повторять условия из CandleFactory либо добавить Candle.LastTradeId - и фильтровать по нему

            return false;
        }
        
        private void LoadNextCandle()
        {
            // Сделано для повышения производительности - чтобы не обращаться каждый раз к массиву
            if (_dayCandles.Count() < _nextCandlePos + 1)
            {
                _nextFistCandle = null;
                _nextSecondCandle = null;
                return;
            }

            _nextFistCandle = _dayCandles.ElementAt(_nextCandlePos);

            if (_dayCandles.Count() < _nextCandlePos + 2)
                _nextSecondCandle = null;
            else
                _nextSecondCandle = _dayCandles.ElementAt(_nextCandlePos + 1);

            _nextCandlePos++;
        }

        private void OnNewTrades(IEnumerable<Trade> trades)
        {
            foreach (var token in _tokens)
            {
                foreach (var trade in trades)
                    ProcessNextCandles(token, trade);
            }           
        }
        
        void IDisposable.Dispose()
        {
            Trader.NewTrades -= OnNewTrades;            
        }
    }
}
